@@ -1,6 +1,6 @@
 /* pinentry-gtk-2.c
    Copyright (C) 1999 Robert Bihlmeyer <robbe@orcus.priv.at>
-   Copyright (C) 2001, 2002 g10 Code GmbH
+   Copyright (C) 2001, 2002, 2007 g10 Code GmbH
    Copyright (C) 2004 by Albrecht Dre <albrecht.dress@arcor.de>
 
    pinentry-gtk-2 is a pinentry application for the Gtk+-2 widget set.
@@ -56,13 +56,22 @@ static int passphrase_ok;
 static int confirm_yes;
 
 static GtkWidget *entry;
+static GtkWidget *qualitybar;
 static GtkWidget *insure;
 static GtkWidget *time_out;
+static GtkTooltips *tooltips;
 
 /* Gnome hig small and large space in pixels.  */
 #define HIG_SMALL      6
 #define HIG_LARGE     12
 
+/* The text shown in the quality bar when no text is shown.  This is not
+ * the empty string, becase because with an empty string the height of
+ * the quality bar is less than with a non-empty string.  This results
+ * in ugly layout changes when the text changes from non-empty to empty
+ * and vice versa */
+#define QUALITYBAR_EMPTY_TEXT " "
+
 
 /* Constrain size of the window the window should not shrink beyond
    the requisition, and should not grow vertically.  */
@@ -181,15 +190,60 @@ pinentry_utf8_validate (gchar *text)
 }
 
 
+/* Handler called for "changed".   We use it to update the quality
+   indicator.  */
+static void
+changed_text_handler (GtkWidget *widget)
+{
+  char textbuf[50];
+  const char *s;
+  int length;
+  int percent;
+  GdkColor color = { 0, 0, 0, 0};
+
+  if (!qualitybar || !pinentry->quality_bar)
+    return;
+
+  s = gtk_secure_entry_get_text (GTK_SECURE_ENTRY (widget));
+  if (!s)
+    s = "";
+  length = strlen (s);
+  percent = length? pinentry_inq_quality (pinentry, s, length) : 0;
+  if (!length)
+    {
+      strcpy(textbuf, QUALITYBAR_EMPTY_TEXT);
+      color.red = 0xffff;
+    }
+  else if (percent < 0)
+    {
+      snprintf (textbuf, sizeof textbuf, "(%d%%)", -percent);
+      color.red = 0xffff;
+      percent = -percent;
+    }
+  else
+    {
+      snprintf (textbuf, sizeof textbuf, "%d%%", percent);
+      color.green = 0xffff;
+    }
+  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (qualitybar), 
+                                 (double)percent/100.0);
+  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (qualitybar), textbuf);
+  gtk_widget_modify_bg (qualitybar, GTK_STATE_PRELIGHT, &color);
+}
+
+
+
 static GtkWidget *
 create_window (int confirm_mode)
 {
   GtkWidget *w;
-  GtkWidget *win, *box, *ebox;
+  GtkWidget *win, *box;
   GtkWidget *wvbox, *chbox, *bbox;
   GtkAccelGroup *acc;
   gchar *msg;
 
+  tooltips = gtk_tooltips_new ();
+
   /* FIXME: check the grabbing code against the one we used with the
      old gpg-agent */
   win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -237,7 +291,7 @@ create_window (int confirm_mode)
       g_free (msg);
       gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
       gtk_label_set_line_wrap (GTK_LABEL (w), TRUE);
-	gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
+      gtk_box_pack_start (GTK_BOX (box), w, TRUE, FALSE, 0);
     }
   if (pinentry->error && !confirm_mode)
     {
@@ -252,26 +306,56 @@ create_window (int confirm_mode)
       gtk_widget_modify_fg (w, GTK_STATE_NORMAL, &color);
     }
 
-  ebox = gtk_hbox_new (FALSE, HIG_SMALL);
-  gtk_box_pack_start (GTK_BOX(box), ebox, FALSE, FALSE, 0);
+  qualitybar = NULL;
 
   if (!confirm_mode)
     {
+      GtkWidget* table = gtk_table_new (pinentry->quality_bar ? 2 : 1, 2,
+					FALSE);
+      gtk_box_pack_start (GTK_BOX (box), table, FALSE, FALSE, 0);
+
       if (pinentry->prompt)
 	{
 	  msg = pinentry_utf8_validate (pinentry->prompt);
 	  w = gtk_label_new (msg);
 	  g_free (msg);
-	  gtk_box_pack_start (GTK_BOX (ebox), w, FALSE, FALSE, 0);
+	  gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
+	  gtk_table_attach (GTK_TABLE (table), w, 0, 1, 0, 1,
+			    GTK_FILL, GTK_FILL, 4, 0);
 	}
+
       entry = gtk_secure_entry_new ();
       gtk_widget_set_size_request (entry, 200, -1);
       g_signal_connect (G_OBJECT (entry), "activate",
 			G_CALLBACK (enter_callback), entry);
-      gtk_box_pack_start (GTK_BOX (ebox), entry, TRUE, TRUE, 0);
+      g_signal_connect (G_OBJECT (entry), "changed",
+                        G_CALLBACK (changed_text_handler), entry);
+      gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
+                        GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 0);
       gtk_widget_grab_focus (entry);
       gtk_widget_show (entry);
 
+      if (pinentry->quality_bar)
+	{
+          msg = pinentry_utf8_validate (pinentry->quality_bar);
+	  w = gtk_label_new (msg);
+          g_free (msg);
+	  gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
+	  gtk_table_attach (GTK_TABLE (table), w, 0, 1, 1, 2,
+			    GTK_FILL, GTK_FILL, 4, 0);
+	  qualitybar = gtk_progress_bar_new();
+	  gtk_widget_add_events (qualitybar,
+				 GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+	  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (qualitybar),
+				     QUALITYBAR_EMPTY_TEXT);
+	  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (qualitybar), 0.0);
+          if (pinentry->quality_bar_tt)
+            gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltips), qualitybar,
+                                  pinentry->quality_bar_tt, "");
+	  gtk_table_attach (GTK_TABLE (table), qualitybar, 1, 2, 1, 2,
+	  		    GTK_EXPAND|GTK_FILL, GTK_EXPAND|GTK_FILL, 0, 0);
+	}
+
       if (pinentry->enhanced)
 	{
 	  GtkWidget *sbox = gtk_hbox_new (FALSE, HIG_SMALL);
@@ -304,19 +388,22 @@ create_window (int confirm_mode)
   gtk_box_set_spacing (GTK_BOX (bbox), 6);
   gtk_box_pack_start (GTK_BOX (wvbox), bbox, TRUE, FALSE, 0);
 
-  if (pinentry->cancel)
+  if (!pinentry->one_button)
     {
-      msg = pinentry_utf8_validate (pinentry->cancel);
-      w = gtk_button_new_with_label (msg);
-      g_free (msg);
+      if (pinentry->cancel)
+        {
+          msg = pinentry_utf8_validate (pinentry->cancel);
+          w = gtk_button_new_with_label (msg);
+          g_free (msg);
+        }
+      else
+        w = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+      gtk_container_add (GTK_CONTAINER (bbox), w);
+      g_signal_connect (G_OBJECT (w), "clicked",
+                        G_CALLBACK (confirm_mode ? confirm_button_clicked
+                                    : button_clicked), NULL);
+      GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
     }
-  else
-    w = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-  gtk_container_add (GTK_CONTAINER (bbox), w);
-  g_signal_connect (G_OBJECT (w), "clicked",
-		    G_CALLBACK (confirm_mode ? confirm_button_clicked
-				: button_clicked), NULL);
-  GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
   
   if (pinentry->ok)
     {
